/***************************************************************************
                                QSSegment.h                              
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef QSSEGMENT_H
#define QSSEGMENT_H

#include"qscoord.h"
#include"qsgattr.h"
#include"qsmatrix.h"

class QSDrv;
class QSCurve;
class QSAxes;

/**
  * Draws a single point from curve.
  * @see QSSamples
  * @see QSBars
  * @author Kamil Dobkowski
  */
class QSSegment {

  public:
   /**
     * Constructor.
     */
   QSSegment();
   /**
     * Destructor
     */
   virtual ~QSSegment();
   /**
     * Returns a number of passes.
     */
   virtual int startDraw( const QSCurve * );
   /**
     * Inits pass
     */
   virtual void initPass( int pass );
   /**
     * Draws a single point of the data
     */
   virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 ) = 0;
   /**
     * Ends pass
     */
   virtual void endPass() {}
   /**
     * Ends drawing
     */
   virtual void stopDraw() {}

  protected:
    QSDrv *m_drv;
    const QSCurve *m_parent;
  };

//-------------------------------------------------------------//

/**
  * Draws lines.
  * @author Kamil Dobkowski
  */
class QSLines : public QSSegment  {

  public:

     QSLines();
     virtual ~QSLines();
     virtual int startDraw( const QSCurve *parent );
     virtual void initPass( int pass );
     virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 );
     virtual void endPass();
     virtual void stopDraw();

  private:

    QSGLine m_curr_line;
};

//-------------------------------------------------------------//

/**
  * Draws polygons.
  * @author Kamil Dobkowski
  */
class QSPolys : public QSSegment  {

  public:

     enum Style { Area = 0, Ribbon = 1 };

     QSPolys( Style style );
     virtual ~QSPolys();
     virtual int startDraw( const QSCurve *parent );
     virtual void initPass( int pass );
     virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 );
     virtual void endPass();
     virtual void stopDraw();

  private:
	
    Style m_style;
    double m_zero_level;
    QSPt2f m_prev1;
    QSPt2f m_prev2;
    int m_pass;
    QSGLine m_curr_line;
    QSGFill m_curr_fill;
};

//-------------------------------------------------------------//

/**
  * Draws bars.
  * @author Kamil Dobkowski
  */
class QSBars : public QSSegment  {

  public:
	
     QSBars();
     virtual ~QSBars();
     virtual int startDraw( const QSCurve *parent );
     virtual void initPass( int pass );
     virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 );
     virtual void endPass();
     virtual void stopDraw();

  private:

    double m_zero_level;
    double m_prevx;
    QSPt2f m_prev;
    QSGLine m_curr_line;
    QSGFill m_curr_fill;
    void draw_bar( int nr, const QSPt2f& pos );
};

//-------------------------------------------------------------//

/**
  * Draws vectors, ellipses, rectangles, flux.
  * @author Kamil Dobkowski
  */
class QSFigures: public QSSegment  {

  public:
	
   enum Style { Vectors = 0, Rectangles = 1, Ellipses = 2, Flux = 3 };

   QSFigures( Style style );
   virtual ~QSFigures();
   virtual int startDraw( const QSCurve *parent );
   virtual void initPass( int pass );
   virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 );
   virtual void endPass();
   virtual void stopDraw();

  private:
	int m_style;
  };

//-------------------------------------------------------------//

/**
  * Draws Stairs.
  * @author Kamil Dobkowski
  */
class QSStairs : public QSSegment  {

  public:

     enum Style { Left, Middle, Right };

     QSStairs( Style style );
     virtual ~QSStairs();
     virtual int startDraw( const QSCurve *parent );
     virtual void initPass( int pass );
     virtual void drawSegment( int nr, const QSPt2f& point, const QSPt2f& delta, const QSGLine& l, const QSGFill& f, const QSGArrow& a1, const QSGArrow& a2 );
     virtual void endPass();
     virtual void stopDraw();

  private:
    Style m_style;
    QSPt2f m_prev;
    QSPt2f m_prev_prev;
    QSGLine curr_line;
    void get_stair( const QSPt2f& pos, QSPt2f pos[3] );
};

#endif








