/***************************************************************************
                              qssurface.cpp
                             -------------------
    begin                : 01-January-2000

    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@poczta.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<stdio.h>
#include<math.h>
#include<assert.h>
#include<qapplication.h>
#include"qssurface.h"
#include"qsdrv.h"

/*
 We don't use sorting do use painter algorithm to hide invisible surfaces but rather
 divide all mesh into four quarters. Let's look at our surface form the top:

      X or i or cols
     -->      <--
    +------+------+  |
  | |             |  |
  | |  0      1   | \/
 \/ |      M      |
    +      +      +     Y or j or rows
  ^ |             |  ^
  | |  2      3   |  |
  | |             |  |
    +------+------+
     -->      <--

  0,1,2,3 - d->quarter.number
  --> ...   d->quatrer.di, quarter.dj ( drawing direction )
  M - quarter.middle

  Notice that all four quarters are nessesary only when perspective is on. In other
  case only quarter 0 is drawn ( it covers the whole surface ).
*/

struct QSSurface::surface_runtime_data {
	QSPt2 middle;
	QSPt2 min;
	QSPt2 max;
	QSPt2 delta;
	bool x_reversed;
	bool y_reversed;
	int curr_col;
	int curr_row;
	
        struct quarter_t {
            bool empty;
            int  number;
            int  iMin, iMax;
            int  jMin, jMax;
            int  di, dj;
	    int  ci, cj;
	    int  p0, p1, p2, p3;
           } quarter;
	
        int w;
        int h;
	int pi;
	int pj;
	QSAxis *xaxis;
        QSAxis *yaxis;
        QSAxis *zaxis;
        QSAxis *vaxis;

        bool is_x_index;
        bool is_y_index;
        bool is_4d_data;
	QSMatrix *x_vector;
	QSMatrix *y_vector;
	QSMatrix *z_data;
        QSMatrix *v_data;

	QSPt3f  *cvbuff;  // current row vertex buffer
	QSPt3f  *nvbuff;  // next row vertex buffer
	QSPt3f  *pnbuff;  // previous row normal buffer
	QSPt3f  *cmbuff;  // current row mesh normal buffer
	QSGFill *pfbuff;  // previous row fill buffer

	QSPt3f  mvbuff[4];  // mesh vertex buffer
	QSPt3f  tvbuff[4];  // temporary buffer
	QSGFill mfbuff[4];  // mesh vertex fill buffer
	QSPt3f  mnbuff[5];  // mesh vertex normal buffer
	
        QSPt3f  pvbuff[2];
	QSPt3f  pmbuff[2];

	double v_buff[4];

	QSGPoint point_mark;

       	inline double xvector( int col );
	inline double yvector( int row );
	inline double data( int row, int col );
	inline double vdata( int row, int col );

        enum ColorMode { AutoColor, OneColor, GrayColor, RGBColor, RGBAColor } color_mode;
   };


//-------------------------------------------------------------//

QSSurface::QSSurface(QSAxes* parent, const char * name)
:QSPlot3D(parent,name)
  {
   assert( parent );

   m_minmax_z_valid = false;
   m_minmax_v_valid = false;
   m_min_z = 0.0;
   m_max_z = 0.0;
   m_min_v = 0.0;
   m_max_v = 0.0;
   m_title_str = tr("Untitled surface");

   #define CHANNELS_NUM	8
   initChannelTable( CHANNELS_NUM );
  }

//-------------------------------------------------------------//

QSSurface::~QSSurface()
  {
  }

//-------------------------------------------------------------//

void QSSurface::dataChanged( int channel )
// Ouu. We need to calculate new extremes in data
// Refresh data on screen also.
  {
   if ( channel == ZData  || channel == -1 ) m_minmax_z_valid = false;
   if ( channel == VData || channel == -1 ) m_minmax_v_valid = false;
   QSPlot3D::dataChanged( channel );
  }

//-------------------------------------------------------------//

void QSSurface::allocRuntimeData()
 {
  QSPlot3D::allocRuntimeData();
  d = new surface_runtime_data();
  d->cvbuff  = NULL;
  d->nvbuff  = NULL;
  d->pfbuff  = NULL;
  d->pnbuff  = NULL;
  d->cmbuff  = NULL;
  d->curr_row = -9999;
  d->curr_col = -9999;
  d->x_vector = matrix(XVector);
  d->y_vector = matrix(YVector);
  d->z_data   = matrix(ZData);
  d->v_data   = matrix(VData);
  d->xaxis = defaultAxis(QSAxis::XAxisType);
  d->yaxis = defaultAxis(QSAxis::YAxisType);
  d->zaxis = defaultAxis(QSAxis::ZAxisType);
  d->vaxis = defaultAxis(QSAxis::VAxisType);
  d->w = matrixCols( ZData );
  d->h = matrixRows( ZData );
  d->is_x_index = false;
  d->is_y_index = false;
  d->is_4d_data = false;
  if ( matrixCols(XVector) == d->w ) d->is_x_index = true;
  if ( matrixRows(YVector) == d->h ) d->is_y_index = true;
  if ( matrixRows(VData) == d->h &&
       matrixCols(VData) == d->w   ) d->is_4d_data = true;
  d->point_mark = m_settings.points[PointMark];
  init_colors();
 }

//-------------------------------------------------------------//

void QSSurface::freeRuntimeData()
 {
  free_buffers();
  delete d; d=NULL;
  QSPlot3D::freeRuntimeData();
 }

 //-------------------------------------------------------------//
 //-------------------------------------------------------------//
 //-------------------------------------------------------------//

bool QSSurface::getAxisRange( QSAxis *axis, double& min, double& max )
 {
  allocRuntimeData();
  if ( d->h == 0 || d->w == 0 ) { freeRuntimeData(); return false; }

  if ( !m_minmax_z_valid ) {
     for ( int j=0;j<d->h;j++ ) {		
	for ( int i=0;i<d->w;i++ ) {		 	
		double z = d->data(j,i);
        	if ( i == 0 && j == 0 ) {
        		 m_max_z = z;
        		 m_min_z = z;
        		} else {
		         m_min_z = QMIN( m_min_z, z );
		 	 m_max_z = QMAX( m_max_z, z );
		 	}
		}
	}
     m_minmax_z_valid = true;
    }

  if ( !m_minmax_v_valid && d->is_4d_data ) {
     for ( int j=0;j<d->h;j++ ) {		
	for ( int i=0;i<d->w;i++ ) {		 	
		double v = d->vdata(j,i);
        	if ( i == 0 && j == 0 ) {
        		 m_max_v = v;
        		 m_min_v = v;
        		} else {
		         m_min_v = QMIN( m_min_v, v );
		 	 m_max_v = QMAX( m_max_v, v );
		 	}
		}
	}
     m_minmax_v_valid = true;
    }

   double xmin = d->xvector(0);
   double xmax = d->xvector(d->w-1);
   double ymin = d->yvector(0);
   double ymax = d->yvector(d->h-1);
   if ( xmin > xmax ) { double xtemp = xmin; xmin = xmax; xmax = xtemp; }
   if ( ymin > ymax ) { double ytemp = ymin; ymin = ymax; ymax = ytemp; }

   if ( axis == d->xaxis ) { min = xmin; max = xmax; }
   else if ( axis == d->yaxis ) { min = ymin; max = ymax; }
   else if ( axis == d->zaxis ) { min = m_min_z; max = m_max_z; }
   else if ( axis == d->vaxis && !d->is_4d_data ) { min = m_min_z; max = m_max_z; }
   else if ( axis == d->vaxis &&  d->is_4d_data ) { min = m_min_v; max = m_max_v; }
   else { freeRuntimeData(); return false; }

   freeRuntimeData();
   return true;
 }


//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

inline double QSSurface::surface_runtime_data::xvector( int col ) {
   return is_x_index ? x_vector->value(0,col) : col;
 }

//-------------------------------------------------------------//

inline double QSSurface::surface_runtime_data::yvector( int row ) {
  return is_y_index ? y_vector->value(row,0) : row;
 }

//-------------------------------------------------------------//

inline double QSSurface::surface_runtime_data::data( int row, int col )
 {
  return z_data->value(row,col);
 }

//-------------------------------------------------------------//

inline double QSSurface::surface_runtime_data::vdata( int row, int col )
 {
  return v_data->value(row,col);
 }

//-------------------------------------------------------------//

bool QSSurface::start()
 {
  QSPlot3D::start();
  if ( d->w == 0 || d->h == 0 ) return false;
  alloc_buffers();

  // check if clipping is needed
  double xmin = d->xvector(0);
  double xmax = d->xvector(d->w-1);
  double ymin = d->yvector(0);
  double ymax = d->yvector(d->h-1);
  if ( xmin > xmax ) { double xtemp = xmin; xmin = xmax; xmax = xtemp; }
  if ( ymin > ymax ) { double ytemp = ymin; ymin = ymax; ymax = ytemp; }

  init_loops();

  d->quarter.number = 0;
  prepare_quarter();
  m_drv->setLine( line(MeshLine) );
  m_drv->setFill( fill(TMeshFill) );
  m_drv->setTopBottom( topBottom() );
  m_drv->setBottomFill( fill(BMeshFill) );
  return true;
 }


//-------------------------------------------------------------//

void QSSurface::init_loops()
 {
  // zero in world coordinates not nessesary means zero index to data matrix...
  // dataToWorld(xvector(0)) is 0 or 1 ?
  d->x_reversed = false;
  d->y_reversed = false;
  if ( d->xaxis->reversed() ) d->x_reversed = !d->x_reversed;
  if ( d->yaxis->reversed() ) d->y_reversed = !d->y_reversed;
  if ( d->xvector(0) > d->xvector(d->w-1) ) d->x_reversed = !d->x_reversed;
  if ( d->yvector(0) > d->yvector(d->h-1) ) d->y_reversed = !d->y_reversed;	

  int i;
  bool is_p_in;
  bool is_c_in;
  double cvalue = 0.0;
  double pvalue = 0.0;

  // visible range in index to ZData
  d->min.x = d->w; // empty area
  d->max.x =   -1;
  for( i=0;i<d->w;i++ ) {
        pvalue = cvalue;
  	cvalue = d->xaxis->dataToWorld(d->xvector(i));
        is_p_in = ( pvalue >= 0.0 && pvalue <= 1.0 && i>0 );
        is_c_in = ( cvalue >= 0.0 && cvalue <= 1.0 && i>0 );
        if ( is_c_in || is_p_in ) {
        	d->min.x = QMIN( d->min.x, i-1 );
        	d->max.x = QMAX( d->max.x, i );
        	}
        if ( i>0 && pvalue < 0.0 && cvalue > 1.0 ||
             i>0 && cvalue < 0.0 && pvalue > 1.0 ) {
             	d->min.x = i-1; d->max.x = i;
             	}
        }

  d->min.y = d->h; // empty area
  d->max.y =   -1;
  for( i=0;i<d->h;i++ ) {
        pvalue = cvalue;
  	cvalue = d->yaxis->dataToWorld(d->yvector(i));
        is_p_in = ( pvalue >= 0.0 && pvalue <= 1.0 && i>0 );
        is_c_in = ( cvalue >= 0.0 && cvalue <= 1.0 && i>0 );
        if ( is_c_in || is_p_in ) {
        	d->min.y = QMIN( d->min.y, i-1 );
        	d->max.y = QMAX( d->max.y, i );
        	}
	if ( i>0 && pvalue < 0.0 && cvalue > 1.0 ||
             i>0 && cvalue < 0.0 && pvalue > 1.0 ) {
             	d->min.y = i-1;
             	d->max.y = i;
             	}
        }

  // delta must be set as in the first quarter
  d->delta.x = 1;
  d->delta.y = 1;

  // start position - which index to data matrix has the furthest point
  QSPt3f f = m_axes->proj()->furthest();
  bool fx_at_zero = (f.x<0.5); if ( d->x_reversed ) fx_at_zero = !fx_at_zero;
  bool fy_at_zero = (f.y<0.5); if ( d->y_reversed ) fy_at_zero = !fy_at_zero;
  if ( !fx_at_zero ) d->delta.x = -d->delta.x;
  if ( !fy_at_zero ) d->delta.y = -d->delta.y;

  // middle point
  // middle x must be a valid index
  QSPt2f middle = m_axes->proj()->middle(); // in world coordinates
  for( i=1;i<d->w;i++) {
  	double value = d->xaxis->dataToWorld(d->xvector(i));
        if (  d->x_reversed && value < middle.x ) break;
        if ( !d->x_reversed && value > middle.x ) break;
        }
  d->middle.x = d->delta.x > 0 ? i-1 : i; // add the crossing mesh to the later drawn quarter
  for( i=1;i<d->h;i++) {
  	double value = d->yaxis->dataToWorld(d->yvector(i));
        if (  d->y_reversed && value < middle.y ) break;
        if ( !d->y_reversed && value > middle.y ) break;
        }
  d->middle.y = d->delta.y > 0 ? i-1 : i;
  }

//-------------------------------------------------------------//

void QSSurface::prepare_quarter()
// divide graph into four parts with different drawing directions
// to be sure that invisible areas are hidden ( further objects are
// drawn first ).
  {
   // empty quarter
   d->quarter.empty = true;
   ///if ( !m_axes->perspective() && d->quarter.number > 0 ) return;
   d->quarter.iMin = d->min.x;
   d->quarter.iMax = d->max.x;
   d->quarter.jMin = d->min.y;
   d->quarter.jMax = d->max.y;
   d->quarter.di   = d->delta.x;
   d->quarter.dj   = d->delta.y;
   switch( d->quarter.number ) {
   	 case 0: break;
   	 case 1: d->quarter.di = -d->quarter.di; break;
   	 case 2: d->quarter.dj = -d->quarter.dj; break;
   	 case 3: d->quarter.di = -d->quarter.di; d->quarter.dj = -d->quarter.dj; break;
   	 default: break;
   	}

   if ( d->quarter.di > 0 ) d->quarter.iMax = d->middle.x; else d->quarter.iMin = d->middle.x;
   if ( d->quarter.dj > 0 ) d->quarter.jMax = d->middle.y; else d->quarter.jMin = d->middle.y;

   // reverse the scan direction for OpenGL ( more effective )
   if ( m_corder == QSDrv::NearerFirst ) {
        d->quarter.di = -d->quarter.di;
        d->quarter.dj = -d->quarter.dj;
       }

    if ( (d->quarter.iMax-d->quarter.iMin) < 1 || (d->quarter.jMax-d->quarter.jMin) < 1 ) return;

   // a to po co ?
   d->quarter.iMin -= QMAX(d->delta.x,-d->delta.x);
   d->quarter.iMax += QMAX(d->delta.x,-d->delta.x);
   d->quarter.jMin -= QMAX(d->delta.y,-d->delta.y);
   d->quarter.jMax += QMAX(d->delta.y,-d->delta.y);
            	
   // start positions
   if ( d->quarter.di > 0 ) d->quarter.ci = d->quarter.iMin; else d->quarter.ci = d->quarter.iMax;
   if ( d->quarter.dj > 0 ) d->quarter.cj = d->quarter.jMin; else d->quarter.cj = d->quarter.jMax;

   // vertex ordering for normal calculations
   d->quarter.p0 = 3; d->quarter.p1 = 2; d->quarter.p2 = 1; d->quarter.p3 = 0;
   if ( (d->quarter.di<0) != d->x_reversed ) { int temp = d->quarter.p0; d->quarter.p0 = d->quarter.p1; d->quarter.p1 = temp; temp = d->quarter.p3; d->quarter.p3 = d->quarter.p2; d->quarter.p2 = temp; }
   if ( (d->quarter.dj<0) != d->y_reversed ) { int temp = d->quarter.p0; d->quarter.p0 = d->quarter.p3; d->quarter.p3 = temp; temp = d->quarter.p1; d->quarter.p1 = d->quarter.p2; d->quarter.p2 = temp; }

   d->pi = 0; d->pj = 0;
   d->quarter.empty = false;
  }


   // middle point
   //int middle_x = ( d->middle.x / d->delta.x ) * d->delta.x;
   //int middle_y = ( d->middle.y / d->delta.y ) * d->delta.y;
   //if ( m_axes->perspective() ) if ( d->quarter.di > 0 ) d->quarter.iMax = middle_x; else d->quarter.iMin = middle_x;
   //if ( m_axes->perspective() ) if ( d->quarter.dj > 0 ) d->quarter.jMax = middle_y; else d->quarter.jMin = middle_y;

//-------------------------------------------------------------//

void QSSurface::end()
 {
  free_buffers();
  QSPlot3D::end();
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//



//-------------------------------------------------------------//

bool QSSurface::step()
//
//       -------- --------- --------
//      |        |         |        |
//      |        |         |        |
// pvb   ---- mvb[p0] -- mvb[p1] ---              ---> di    previous
//      |        | / / / / |        |        |
//      |        |/ / / / /|        |        |
// cvb   ---- mvb[p3] -- mvb[p2] ---        \ / dj           current              row
// 	|        |         |        |
//      |        |         |        |
// nvb   -------- --------- --------                         next
//                            (quarter.ci, quarter.cj)
//			  pi-1	   pi
//
//		 |  pi-1   |   pi   | for mesh normal buffers
//                    1        0
//                        col
// mvbuff[quarter.p2] is the currently drawn point
// Must rewrite this mess once again
 {
  int curr_step = 0;

  if ( !d->quarter.empty )
  while ( d->quarter.cj >= d->quarter.jMin && d->quarter.cj <= d->quarter.jMax  ) {
    while ( d->quarter.ci >= d->quarter.iMin && d->quarter.ci <= d->quarter.iMax  ) {

        //
        // The point currently drawn ( see m..buff[quarter.p2] )
        //
        int row = d->quarter.cj - d->quarter.dj;
	int col = d->quarter.ci - d->quarter.di;

	// remember two previous values in the current row
	d->pvbuff[1]  = d->pvbuff[0];
	d->pvbuff[0]  = d->cvbuff[d->pi];
	// assign value
        d->cvbuff[d->pi] = d->nvbuff[d->pi];

	if ( m_cnormals == QSDrv::VertexNormals ) {
        	d->pmbuff[1] = d->pmbuff[0];
		d->pmbuff[0] = d->cmbuff[d->pi];
        	d->cmbuff[d->pi].set( 0.0, 0.0, 0.0 ); // no normal
		}

	//
	//  prepare the next row
	//
	if ( d->quarter.ci >= 0 && d->quarter.ci < d->w &&
	     d->quarter.cj >= 0 && d->quarter.cj < d->h ) {
	        QSPt3f dpos( d->xvector(d->quarter.ci), d->yvector(d->quarter.cj), d->data(d->quarter.cj,d->quarter.ci) );
		d->nvbuff[d->pi] = dataToWorld(dpos);
              // normal
              if ( m_cnormals == QSDrv::VertexNormals )
	      if ( col >= 0 && col < d->w &&
	           row >= 0 && row < d->h ) {
		 d->tvbuff[d->quarter.p0] = d->cvbuff[d->pi-1];
                 d->tvbuff[d->quarter.p1] = d->cvbuff[d->pi-0];
                 d->tvbuff[d->quarter.p2] = d->nvbuff[d->pi-0];
                 d->tvbuff[d->quarter.p3] = d->nvbuff[d->pi-1];
		 d->cmbuff[d->pi] = m_axes->proj()->normal( d->tvbuff );
		}
	    }
	//
	// Rotate buffers
	//
	if ( d->pi > 0 ) {
		// rotate buffers
        	d->mvbuff[d->quarter.p0] = d->mvbuff[d->quarter.p1];
		d->mvbuff[d->quarter.p3] = d->mvbuff[d->quarter.p2];
        	d->mvbuff[d->quarter.p2] = d->cvbuff[d->pi-1];
        	d->mvbuff[d->quarter.p1] = d->pvbuff[1];
		
		if ( m_ccolors == QSDrv::VertexColors ) {
			d->mfbuff[d->quarter.p0] = d->mfbuff[d->quarter.p1];
			d->mfbuff[d->quarter.p3] = d->mfbuff[d->quarter.p2];
        		d->mfbuff[d->quarter.p1] = d->pfbuff[d->pi-1];
			d->pfbuff[d->pi-1] = vertex_color( col, row );
                        d->mfbuff[d->quarter.p2] = d->pfbuff[d->pi-1];
			} else {
			d->mfbuff[0] = vertex_color( col, row );
			}

		if ( m_cnormals == QSDrv::VertexNormals ) {
                        d->mnbuff[d->quarter.p0+1] = d->mnbuff[d->quarter.p1+1];
			d->mnbuff[d->quarter.p3+1] = d->mnbuff[d->quarter.p2+1];
        		d->mnbuff[d->quarter.p1+1] = d->pnbuff[d->pi-1];
			d->pnbuff[d->pi-1] = vertex_normal();
			d->mnbuff[d->quarter.p2+1] = d->pnbuff[d->pi-1];
			}
               }

	//
	// Draw mesh
	//
	if ( d->pi > 2 && d->pj > 2 ) {
	     if ( m_cnormals == QSDrv::MeshNormal ) d->mnbuff[0] = m_axes->proj()->normal( d->mvbuff );
	     else if ( m_cnormals == QSDrv::VertexNormals ) d->mnbuff[0] = d->pmbuff[1];

	     double *v_buff = NULL;	
	     if ( d->is_4d_data ) {
                d->v_buff[d->quarter.p0] = d->vaxis->dataToWorld( d->vdata( row - d->quarter.dj, col - d->quarter.di ) );
		d->v_buff[d->quarter.p1] = d->vaxis->dataToWorld( d->vdata( row - d->quarter.dj, col ) );
		d->v_buff[d->quarter.p2] = d->vaxis->dataToWorld( d->vdata( row, col ) );
                d->v_buff[d->quarter.p3] = d->vaxis->dataToWorld( d->vdata( row, col - d->quarter.di ) );
		v_buff = d->v_buff;
		}
	     drawPolygon( d->mvbuff, 4, d->mnbuff, d->mfbuff, v_buff );
            }

        // points on quartr edges can be drawn two times
	if ( d->point_mark.style != QSGPoint::Invisible ) {
	    // when user defined colors apply it also to
	    // points
	    if ( d->color_mode == surface_runtime_data::RGBColor  ||
		 d->color_mode == surface_runtime_data::GrayColor ||
		 d->color_mode == surface_runtime_data::RGBAColor )
	         if ( m_ccolors == QSDrv::VertexColors ) d->point_mark.color = d->mfbuff[d->quarter.p2].color;
						    else d->point_mark.color = d->mfbuff[0].color;
            if ( m_axes->proj()->clipPoint3(d->mvbuff[d->quarter.p2]) ) {
	    	m_drv->drawPoint3( d->mvbuff[d->quarter.p2], d->point_mark );
	    	m_drv->setLine( m_settings.lines[MeshLine] );
            	m_drv->setFill( m_settings.fills[TMeshFill] );
		}
	   }

        d->quarter.ci += d->quarter.di; d->pi ++;

        // end with this little part ...
        if ( curr_step++ > work_steps && m_bkg_handler ) return true;
       }

     d->quarter.cj += d->quarter.dj;
     d->quarter.ci  = d->quarter.di>0 ? d->quarter.iMin : d->quarter.iMax;
     d->pi = 0; d->pj ++;
    }

   if ( d->quarter.number < 3 ) {
                 d->quarter.number += 1;
                 prepare_quarter();
                 return true;
                }

  return false;
 }

//-------------------------------------------------------------//

QSPt3f QSSurface::vertex_normal()
 {
  QSPt3f n;
  n = n + d->pmbuff[1];
  n = n + d->pmbuff[0];
  n = n + d->cmbuff[d->pi-1];
  n = n + d->cmbuff[d->pi-0];
  return m_axes->proj()->normalize( n );
 }


//-------------------------------------------------------------//

void QSSurface::alloc_buffers()
 {
  free_buffers();
  int cols = d->w;
  //if ( m_method == PreviewMethod ) cols = d->w/max(1,cols/m_preview_nx)+1; else cols = d->w;

  d->cvbuff = new QSPt3f[cols+2];
  d->nvbuff = new QSPt3f[cols+2];
  if ( m_ccolors  == QSDrv::VertexColors  ) {
	d->pfbuff = new QSGFill[cols+2];
	}
  if ( m_cnormals == QSDrv::VertexNormals ) {
	d->pnbuff = new QSPt3f[cols+2];
  	d->cmbuff = new QSPt3f[cols+2];
	}
 }

//-------------------------------------------------------------//

void QSSurface::free_buffers()
 {
  delete[] d->cvbuff;
  delete[] d->nvbuff;
  delete[] d->pfbuff;
  delete[] d->pnbuff;
  delete[] d->cmbuff;
  d->cvbuff = NULL;
  d->nvbuff = NULL;
  d->pfbuff = NULL;
  d->pnbuff = NULL;
  d->cmbuff = NULL;
 }

 //-------------------------------------------------------------//

void QSSurface::init_colors()
 {
  int w = matrixCols( ZData );
  int h = matrixRows( ZData );

  d->color_mode = surface_runtime_data::AutoColor;

  if ( colored() == false ) {
           d->color_mode = surface_runtime_data::OneColor;
           return;
          }

  if ( isChannel( ColorsR ) )
           if ( matrixCols( ColorsR ) == w &&
                matrixRows( ColorsR ) == h ) d->color_mode = surface_runtime_data::GrayColor;

 if ( isChannel( ColorsG ) &&
      isChannel( ColorsB ) &&
      d->color_mode == surface_runtime_data::GrayColor )
           if ( matrixCols( ColorsG ) == w &&
                matrixRows( ColorsG ) == h &&
                matrixCols( ColorsB ) == w &&
                matrixRows( ColorsB ) == h ) d->color_mode = surface_runtime_data::RGBColor;

  if ( isChannel( ColorsA ) &&
       d->color_mode ==surface_runtime_data::RGBColor )
           if ( matrixCols( ColorsA ) == w &&
                matrixRows( ColorsA ) == h ) d->color_mode = surface_runtime_data::RGBAColor;

 }

//-------------------------------------------------------------//

QSGFill QSSurface::vertex_color( int i, int j  )
 {
  QSGFill f;

  switch( d->color_mode ) {

    case surface_runtime_data::OneColor:
    		    f = m_settings.fills[TMeshFill];
                    break;

    case surface_runtime_data::AutoColor:
    		    if ( m_ccolors == QSDrv::VertexColors ) m_gradient.fill( d->mvbuff[d->quarter.p2].z, f );
						        else m_gradient.fill( (d->mvbuff[0].z+d->mvbuff[1].z+d->mvbuff[2].z+d->mvbuff[3].z)/4.0, f );
                    break;

    case surface_runtime_data::GrayColor:
    		    f.color.r = f.color.g = f.color.b = (unsigned char )value( j, i, ColorsR ); //c
                    break;

    case surface_runtime_data::RGBColor:
    		    f.color.r = (unsigned char )value( j, i, ColorsR ); //c
                    f.color.g = (unsigned char )value( j, i, ColorsG ); //c
                    f.color.b = (unsigned char )value( j, i, ColorsB ); //c
                    break;

    case surface_runtime_data::RGBAColor:
    		    f.color.r = (unsigned char )value( j, i, ColorsR );
                    f.color.g = (unsigned char )value( j, i, ColorsG );
                    f.color.b = (unsigned char )value( j, i, ColorsB );
                    f.color.a = (unsigned char )value( j, i, ColorsA );
                    break;

    default:        break;

   } 	

  return f;
 }

//-------------------------------------------------------------//

QString QSSurface::posInfo( QSPt2f& pos )
 {
  if ( m_busy ) return QString::null;
  allocRuntimeData();
  if ( d->w == 0 || d->h == 0 ) return QString::null;
  init_loops();

  QSPt2f p;
  double dz = 0.0;
  bool init = false;
  double pdistance = 10.0*10.0;
  QString result = QString::null;
  for ( int row=d->min.y; row<=d->max.y; row++ ) {			
	for ( int col=d->min.x; col<=d->max.x; col++ ) {		 	
		QSPt3f dpos(d->xvector(col),d->yvector(row),d->data(row,col));
		QSPt3f wpos = dataToWorld(dpos);
		QSPt3f cpos = m_proj->world3DToCanvas3(wpos);
		double distance = (pos.x-cpos.x)*(pos.x-cpos.x)+(pos.y-cpos.y)*(pos.y-cpos.y);
		if ( distance < 5.0*5.0 && distance < pdistance && (cpos.z<dz||!init) ) {
			pdistance = distance;
			init = true; dz = cpos.z; p.set(cpos.x,cpos.y);
			result  = QString(tr("row = "))+QString::number(row)+"\n";
			result += QString(tr("col = "))+QString::number(col)+"\n";
			result += QString(tr("X = "))+QString::number(dpos.x)+"\n";
			result += QString(tr("Y = "))+QString::number(dpos.y)+"\n";
			result += QString(tr("Z = "))+QString::number(dpos.z)+"\n";
			if ( d->is_4d_data )
			result += QString(tr("V = "))+QString::number(d->vdata(row,col))+"\n";						
			}
		}
	}
	
  if ( result != QString::null ) pos = p;
  freeRuntimeData();
  return result;
 }

//-------------------------------------------------------------//

QSPt2f QSSurface::legendItemSize( QSDrv *drv )
 {
  return standardLegendItemSize( drv, defaultAxis(QSAxis::ZAxisType), title() );
 }

//-------------------------------------------------------------//

void QSSurface::drawLegendItem( const QSPt2f& pos, QSDrv *drv )
 {
  drawStandardLegendItem( pos, drv, defaultAxis(QSAxis::ZAxisType), title(), &m_gradient );
 }

//-------------------------------------------------------------//

void QSSurface::loadStateFromStream( QDataStream& stream, QSObjectFactory *factory )
 {
  QSPlot3D::loadStateFromStream( stream, factory );
 }

//-------------------------------------------------------------//

void QSSurface::saveStateToStream( QDataStream& stream, QSObjectFactory *factory )
 {
  QSPlot3D::saveStateToStream( stream, factory );
 }

//-------------------------------------------------------------//

QString QSSurface::channelVariable( int channel ) const
 {
  switch( channel ) {
	case ZData:	return "z";
	case XVector:	return "x";
	case YVector:	return "y";
	case VData:	return "v";
	case ColorsR:	return "r";
	case ColorsG:	return "g";
	case ColorsB:	return "b";
	case ColorsA:	return "a";
	}
  return QString::null;
 }

//-------------------------------------------------------------//

QSSurface::ColumnType QSSurface::columnType( int channel, int column ) const
 {
 }


        /**
         * Drawing method.
       	enum DrawMethod { DrawAllMethod, PreviewMethod };
	*/